from .monitor import Monitor


class ConsumerMonitor(Monitor):
    """A consumer monitor that extends base monitor by adding friendly methods to track received records count."""

    def init_info(self):
        super().init_info()
        self.info.setdefault('received_records', 0)
        self._received_records = self.info.get('received_records')

    def received_record(self):
        """Adds to received_records count."""
        self._received_records += 1
        self.info.update({'received_records': self._received_records})



from datetime import timedelta


def get_elapsed_time(start, end):
    """Get elapsed time in a human readable format."""
    td: timedelta = start - end
    return str(timedelta(seconds=td.seconds))



import datetime as dt
import json

from .utils import get_elapsed_time


class Reader:
    _redis_hash_map_prefix = "process_monitor"

    def __init__(self, redis_client):
        assert redis_client.connection_pool.connection_kwargs.get('decode_responses'), \
            "Redis instance must have decode_responses=True"
        self.redis_client = redis_client

    def uptime(self, current_datetime, hash_map):
        iso_start_date = hash_map.pop('start_date', None)
        start_date = dt.datetime.fromisoformat(iso_start_date) if iso_start_date else None
        if start_date is None:
            return start_date, "not supported for this processes"

        uptime_str = get_elapsed_time(current_datetime, start_date)
        return iso_start_date, uptime_str

    def _process_info(self, hash_map):
        current_datetime = dt.datetime.now()
        start_date, uptime = self.uptime(current_datetime, hash_map)
        return {
            **hash_map,
            'start_date': start_date,
            'uptime': uptime,
            'last_signal_age': (
                    current_datetime - dt.datetime.fromisoformat(hash_map.get('last_signal'))
            ).total_seconds(),
            'info': json.loads(hash_map['info'])
        }

    def process_info(self, key):
        return self._process_info(self.redis_client.hgetall(key))

    async def async_process_info(self, key):
        return self._process_info(await self.redis_client.hgetall(key))

    def _get_keys(self):
        return self.redis_client.keys("%s_*" % self._redis_hash_map_prefix)

    def read(self):
        keys = self._get_keys()
        return {
            key.replace("%s_" % self._redis_hash_map_prefix, ""): self.process_info(key)
            for key in keys
        }

    async def async_read(self):
        keys = await self._get_keys()
        return {
            key.replace("%s_" % self._redis_hash_map_prefix, ""): await self.async_process_info(key)
            for key in keys
        }



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))
setup(
    name='process_monitor',
    description='',
    version="0.0.0",
    long_description=open('README.rst').read(),
    author='Mohammed Qudah',
    author_email='mohghq@gmail.com',
    package_dir={'process_monitor': 'process_monitor'},
    packages=find_packages(),
    include_package_data=True,
    install_requires=[],
    tests_require=['tox'],
    classifiers=[],
    scripts=[]
)



import datetime as dt
import functools
import json
import logging

from ..reader import Reader

logger = logging.getLogger('process_monitor')


class Monitor:
    _redis_hash_map_prefix = "process_monitor"

    def __init__(self, redis_client=None):
        self.redis_client = redis_client
        self._thread = None
        self._process_name = None
        self._signal_every = None
        self._received_records = 0
        self._signals_thread_running = False
        self._info = None
        self._start_date = None
        self._asyncio_event_loop = None

    def get_event_loop(self):
        """Create or re-use asyncio event loop."""
        import asyncio
        if self._asyncio_event_loop is None:
            self._asyncio_event_loop = asyncio.get_event_loop()
        return self._asyncio_event_loop

    def get_hash_name(self):
        """Get redis hash map name baed on prefix and process name"""
        return "%s_%s" % (self._redis_hash_map_prefix, self._process_name)

    def _send_signal(self):
        """Send signal with info to redis."""
        pipe = self.redis_client.pipeline()
        pipe.hset(self.get_hash_name(), 'last_signal', str(dt.datetime.now()))
        pipe.hset(self.get_hash_name(), 'info', json.dumps(self.info))
        pipe.execute()
        logger.debug("dispatching signal as [%s]" % self._process_name)

    def _send_signals(self):
        """send signal every N seconds. runs in a thread."""
        import time
        while True:
            self._send_signal()
            time.sleep(self._signal_every)

    def set_uptime(self):
        self.redis_client.hset(self.get_hash_name(), 'start_date', self._start_date.isoformat())

    def run_signals_worker(self, process_name: str, signal_every: int = 10):
        """Dispatch signals every N seconds in a thread.
        :param process_name:
        :param signal_every: a positive int in seconds
        :return:
        """
        import threading
        assert self._signals_thread_running is not True, ".run_signals_worker can be called at most once per monitor."
        assert type(process_name) is str, "expected process_name of type %s but got %s" % (str, type(process_name))
        assert type(signal_every) is int, "expected signal_every of type %s but got %s" % (int, type(signal_every))
        assert signal_every > 0, "signal_every must be greater than 0"
        self._start_date = dt.datetime.now()
        self._process_name = process_name.replace(" ", "_")
        self._signal_every = signal_every
        self.init_info()
        self.set_uptime()
        self._thread = threading.Thread(target=self._send_signals, daemon=True)
        self._thread.start()
        self._signals_thread_running = True

    def init_info(self):
        self.info = json.loads(self.redis_client.hget(self.get_hash_name(), 'info') or '{}')

    @property
    def info(self):
        return self._info

    @info.setter
    def info(self, value):
        assert type(value) is dict, "expected info of type %s but got %s" % (dict, type(value))
        self._info = value
